#!/usr/bin/env python
import config
import capd
import pickle
import sys
import os
import subprocess
import signal
import MySQLdb as mdb
import logging
import logging.handlers

EXIT_CODE = 0



if os.geteuid() == 0:
    import pwd,grp
    target_uid = pwd.getpwnam('nna').pw_uid
    target_gid = grp.getgrnam('nnacmd').gr_gid
    os.setegid(target_gid)
    os.seteuid(target_uid)

logger = logging.getLogger('nnabackend')
logger.setLevel(logging.DEBUG)

handler = logging.handlers.SysLogHandler()
handler.setLevel(logging.ERROR)
logger.addHandler(handler)

handler = logging.FileHandler('/usr/local/nagiosna/var/backend.log')
handler.setLevel(logging.INFO)
logger.addHandler(handler)
        
def start(sourcename):
    try:
        name, directory, port, flowtype, lifetime, sid = get_source(sourcename)
    except Exception, e:
        print str(e)
        raise Exception('Unable to find %s in the database, cannot start it.' % sourcename)
    
    capd.start(name, directory, port, flowtype, lifetime, sid)

def start_all():
    try:
        sources = get_sources()
    except Exception, e:
        print 'Error connecting to the database. %s' % str(e)
        sys.exit(1)
    
    for source in sources:
        name, directory, port, flowtype, lifetime, sid = source
        try:
            capd.start(name, directory, port, flowtype, lifetime, sid)
        except ValueError, e:
            print str(e)
        except Exception, e:
            stop_all()
            raise e

def stop(sourcename):
    try:
        name, directory, port, flowtype, lifetime, sid = get_source(sourcename)
    except Exception, e:
        print str(e)
        raise Exception('Unable to find %s in the database, cannot stop it.' % sourcename)
    
    capd.stop(name, directory, port, flowtype, lifetime, sid)

def stop_all():
    try:
        sources = get_sources()
        return sources
    except Exception, e:
        print 'Error connecting to the database. %s' % str(e)
        sys.exit(1)
    
    for name, directory, port, flowtype, lifetime, sid in sources:
        try:
            capd.stop(name, directory, port, flowtype, lifetime, sid)
        except ValueError, e:
            print str(e)
        except Exception, e:
            print str(e)

def status():
    state = get_pickled_state()
    if state:
        print 'NagiosNA backend is currently running...'
        sys.exit(0)
    else:
        print 'NagiosNA does not appear to be running.'

def get_source(sourcename):
    try:
        source = config.query_database('SELECT `name`, `directory`, `port`, `flowtype`, `lifetime`, `sid` FROM nagiosna_Sources WHERE `name` = "%s"' % sourcename)
    except Exception, e:
        logger.exception(e)
    return source[0]

def get_sources():
    try:
        sources = config.query_database('SELECT `name`, `directory`, `port`, `flowtype`, `lifetime`, `sid` FROM nagiosna_Sources')
    except Exception, e:
        logger.exception(e)
        raise e
    return sources

def main():
    if sys.argv[1] == 'start':
        try:
            servicename = sys.argv[2]
            if not servicename:
                raise IndexError
            start(servicename)
            print '%s process started...' % servicename
        except IndexError:
            start_all()
            print 'NagiosNA backend started...'
    elif sys.argv[1] == 'stop':
        try:
            servicename = sys.argv[2]
            if not servicename:
                raise IndexError
            stop(servicename)
            print '%s process stopped.' % servicename
        except IndexError:
            stop_all()  
            print 'NagiosNA backend stopped.'
    elif sys.argv[1] == 'status':
        try:
            servicename = sys.argv[2]
            if not servicename:
                raise IndexError
            status(servicename)
        except IndexError:
            status()
    sys.exit(0)

if __name__ == '__main__':
    try:
        main()
    except Exception, e:
        print str(e)
        sys.exit(1)
