<?php defined('BASEPATH') OR exit('No direct script access allowed');

class Admin extends NA_Controller
{
    function __construct()
    {
        parent::__construct();

        // Make sure that user is authenticated no matter what page they are on
        if (!$this->ion_auth->logged_in()){
            redirect('login'); // make them log in
        } else {
            if (!$this->ion_auth->is_admin()) {
                redirect(''); // redirect to main dashboard
            }
        }

        $this->page = 'configure';
    }


    public function index()
    {
        $this->init_page('Configure', $this->page);


        $this->data['leftbar'] = $this->load->view('admin/leftbar', array(), true);
        $this->load->view('admin/home', $this->data);
    }

    # License information and set up (get amount of hosts, etc)
    public function license()
    {
        $this->init_page('License Information', $this->page);

        if ($this->input->post('setkey') && !$this->data['demo_mode']) {
            // Set license key if we can
            $key = trim($this->input->post('key'));
            if (is_valid_license_key($key)) {
                set_license_key($key);
            } else {
                $this->data['error'] = "The license key entered was not valid.";
            }
        }

        $this->data['leftbar'] = $this->load->view('admin/leftbar', array("tab" => "license"), true);
        $this->load->view('admin/license', $this->data);
    }

    # User list
    public function users()
    {
        $this->init_page('Users', $this->page);

        $this->data['myuserid'] = $this->session->userdata('user_id');

        //list the users
        $this->data['users'] = $this->ion_auth->users()->result();
        foreach ($this->data['users'] as $k => $user)
        {
            $this->data['users'][$k]->groups = $this->ion_auth->get_users_groups($user->id)->result();
        }

        $this->data['leftbar'] = $this->load->view('admin/leftbar', array("tab" => "users"), true);
        $this->load->view('admin/users', $this->data);
    }

    # Create new user
    public function create_user()
    {
        $this->init_page('Create User', $this->page);

        // Do form validation
        $this->form_validation->set_rules('first_name', $this->lang->line('create_edit_user_fname_val'), 'xss_clean');
        $this->form_validation->set_rules('last_name', $this->lang->line('create_edit_user_lname_val'), 'xss_clean');
        $this->form_validation->set_rules('email', $this->lang->line('create_edit_user_email_val'), 'required|valid_email');
        $this->form_validation->set_rules('phone', $this->lang->line('create_edit_user_phone_val'), 'xss_clean');
        $this->form_validation->set_rules('company', $this->lang->line('create_edit_user_company_val'), 'xss_clean');
        $this->form_validation->set_rules('username', $this->lang->line('create_user_username'), 'required');
        $this->form_validation->set_rules('password', $this->lang->line('create_edit_user_password_val'), 'required|matches[password2]');
        $this->form_validation->set_rules('password2', $this->lang->line('create_edit_user_password_confirm_val'), 'required');
        $this->form_validation->set_rules('apiaccess', $this->lang->line('create_user_apiaccess'), 'required');
        $this->form_validation->set_rules('language', $this->lang->line('language'), 'required');

        $this->data['admin_checked'] = false;
        $this->data['user_checked'] = false;
        $group = $this->input->post('group');
        if ($group == 1) {
            $this->data['admin_checked'] = true;
        } else if ($group == 2) {
            $this->data['user_checked'] = true;
        }

        if (!$this->data['demo_mode']) {
            if ($this->form_validation->run() == true) {
                if (!empty($group)) {
                    $username = $this->input->post('username');
                    $email    = $this->input->post('email');
                    $password = $this->input->post('password');
                    $language = $this->input->post('language');

                    // Generate an API key if they have access
                    $apiaccess = $this->input->post('apiaccess');
                    $apikey = "";
                    if ($apiaccess) {
                        $apikey = sha1(uniqid() . 'nna');
                    }

                    $additional_data = array(
                        'first_name' => $this->input->post('first_name'),
                        'last_name'  => $this->input->post('last_name'),
                        'company'    => $this->input->post('company'),
                        'phone'      => $this->input->post('phone'),
                        'apiaccess'  => $apiaccess,
                        'apikey'     => $apikey,
                        'language'   => $language
                    );

                    // Create the new user
                    $id = $this->ion_auth->register($username, $password, $email, $additional_data);

                    if ($id) {
                        // Add new user to group
                        $this->ion_auth->add_to_group($group, $id);

                        // Finished! Send to main users page
                        redirect('admin/users');
                    } else {
                        $this->data['error'] = "Could not create the user.";
                    }
                } else {
                    $this->data['error'] = "You must select a user access level.";
                }
            } else {
                $this->data['error'] = validation_errors();
            }
        } else { 
            $this->data['error'] = "This action is not available in Demo Mode.";
        }

        $this->data['leftbar'] = $this->load->view('admin/leftbar', array("tab" => "users"), true);
        $this->load->view('admin/create_user', $this->data);
    }

    // Edit users
    public function edit_user($id)
    {
        if (empty($id)) { redirect('admin/users'); }
        $this->init_page('Edit User', $this->page);

        // Grab the user's stuff
        $this->data['user'] = $this->ion_auth->user($id)->row();
        $groups = $this->ion_auth->get_users_groups($id)->result();
        foreach ($groups as $g) {
            if ($g->id == 1) { $this->data['admin_checked'] = true; }
            if ($g->id == 2) { $this->data['user_checked'] = true; }
        }

        // Do form validation stuff
        $this->form_validation->set_rules('first_name', $this->lang->line('create_edit_user_fname_val'), 'xss_clean');
        $this->form_validation->set_rules('last_name', $this->lang->line('create_edit_user_lname_val'), 'xss_clean');
        $this->form_validation->set_rules('email', $this->lang->line('create_edit_user_email_val'), 'required|valid_email');
        $this->form_validation->set_rules('phone', $this->lang->line('create_edit_user_phone_val'), 'xss_clean');
        $this->form_validation->set_rules('company', $this->lang->line('create_edit_user_company_val'), 'xss_clean');
        $this->form_validation->set_rules('password', $this->lang->line('create_edit_user_password_val'), 'matches[password2]');
        $this->form_validation->set_rules('password2', $this->lang->line('create_edit_user_password_confirm_val'), '');
        $this->form_validation->set_rules('apiaccess', $this->lang->line('create_user_apiaccess'), 'required');
        $this->form_validation->set_rules('language', $this->lang->line('language'), 'required');

        $group = $this->input->post('group');
        if ($group == 1) {
            $this->data['admin_checked'] = true;
        } else if ($group == 2) {
            $this->data['user_checked'] = true;
        }

        if (!$this->data['demo_mode']) {
            if ($this->form_validation->run() == true) {
                if (!empty($group) || $id == 1) {

                    // Generate an API key if they have access
                    $apiaccess = $this->input->post('apiaccess');
                    $apikey = "";

                    if ($apiaccess) {
                        $apikey = sha1(uniqid() . 'nna');
                        if ($id == $this->session->userdata('user_id')) {
                            $this->session->set_userdata('apikey', $apikey);
                            $this->session->set_userdata('apiaccess', 1);
                        }
                    } else {
                        if ($id == $this->session->userdata('user_id')) {
                            $this->session->set_userdata('apikey', '');
                            $this->session->set_userdata('apiaccess', 0);
                        }
                    }

                    $additional_data = array(
                        'first_name' => $this->input->post('first_name'),
                        'last_name'  => $this->input->post('last_name'),
                        'company'    => $this->input->post('company'),
                        'phone'      => $this->input->post('phone'),
                        'email'      => $this->input->post('email'),
                        'apiaccess'  => $apiaccess,
                        'apikey'     => $apikey,
                        'ip_address' => '',
                        'language'   => $this->input->post('language')
                    );

                    // Update password
                    $password = $this->input->post('password');
                    if (!empty($password)) {
                        $this->ion_auth_model->reset_password($this->data['user']->username, $password);
                    }

                    // Update user info
                    $this->ion_auth->update($id, $additional_data);

                    // Remove user from all groups and add what they should be in
                    if ($id > 1) {
                        $this->ion_auth->remove_from_group(NULL, $id);
                        $this->ion_auth->add_to_group($group, $id);
                    }

                    $this->session->set_flashdata('msg', "Your changes to the user have been saved.");
                    redirect(current_url());

                } else {
                    $this->data['error'] = "You must select a user access level.";
                }
            } else {
                $this->data['error'] = validation_errors();
            }
        } else { 
            $this->data['error'] = "This action is not available in Demo Mode.";
        }

        $this->data['msg'] = $this->session->flashdata('msg');
        $this->data['leftbar'] = $this->load->view('admin/leftbar', array("tab" => "users"), true);
        $this->load->view('admin/edit_user', $this->data);
    }

    // Delete a user (AJAX called)
    public function delete_user()
    {  
        $id = $this->input->post('id');
        if ($id == 1) {
            $json = array("success" => 0,
                          "errormsg" => "You can not delete the superadmin.");
        } else {
            if ($this->ion_auth->delete_user($id)) {
                $json = array("success" => $val);
            } else {
                $json = array("success" => 0,
                              "errormsg" => "Could not delete that user.");
            }
        }

        print json_encode($json);
    }
}
